@@ -4,33 +4,45 @@
 
 from __future__ import absolute_import, division, print_function
 
-__version__ = "5.4"
+__version__ = "6.0.1"
 __copyright__ = "Copyright (C) 2014-2022 S[&]T, The Netherlands."
 
 __all__ = ["Error", "InternalError", "Struct", "Archive", "open", "config_path"]
 
 import os as _os
 
-from muninn.archive import Archive, create as _create_archive
+from muninn.archive import Archive
 from muninn.exceptions import *
 from muninn.struct import Struct
 
 
 def config_path():
+    """Return the value of the `MUNINN_CONFIG_PATH` environment variable."""
     return _os.environ.get("MUNINN_CONFIG_PATH", "")
 
 
-def open(id="", **kwargs):
-    configuration = {} if not id else _read_archive_config_file(_locate_archive_config_file(id))
-    _merge_configuration(configuration, kwargs)
-    return _create_archive(configuration)
+def open(id=None, **kwargs):
+    """Open an archive for the given archive id, by searching for the
+    corresponding configuration file in the locations found in the
+    `MUNINN_CONFIG_PATH` environment variable.
 
+    Arguments:
+    id  --  Archive id (name of configuration file)
 
-def _merge_configuration(configuration, other_configuration):
-    for name, value in other_configuration.items():
+    Returns:
+    An instance of `muninn.archive.Archive`
+    """
+    if id is None:
+        configuration = {}
+    else:
+        configuration = _read_archive_config_file(_locate_archive_config_file(id))
+
+    for name, value in kwargs.items():
         section = configuration.setdefault(name, {})
         section.update(value)
 
+    return Archive.create(configuration, id)
+
 
 def _read_archive_config_file(path):
     try:
