@@ -9,6 +9,7 @@ from muninn._compat import dictkeys, dictvalues, is_python2_unicode
 import re
 import functools
 import inspect
+import json
 
 try:
     import psycopg2
@@ -408,8 +409,8 @@ class PostgresqlBackend(object):
     def _execute_list(self, sql_list):
         cursor = self._connection.cursor()
         try:
-            for sql in sql_list:
-                cursor.execute(sql)
+            for item in sql_list:
+                cursor.execute(item)
         finally:
             cursor.close()
 
@@ -476,6 +477,9 @@ class PostgresqlBackend(object):
             fields.append("uuid")
             parameters.append(uuid)
 
+        # TODO generic conversion? more overlap between sqlite.py and postgresql.py?
+        parameters = [json.dumps(p) if isinstance(p, dict) else p for p in parameters]
+
         # Build and execute INSERT query.
         query = "INSERT INTO %s (%s) VALUES (%s)" % (self._table_name(name), ", ".join(fields),
                                                      ", ".join([self._placeholder()] * len(fields)))
@@ -513,8 +517,7 @@ class PostgresqlBackend(object):
             except (TypeError, IndexError, AttributeError, KeyError):
                 pass
 
-        if not swallow:
-            raise
+        return swallow
 
     def _link(self, uuid, source_uuids):
         query = "INSERT INTO %s (uuid, source_uuid) SELECT %s, %s" % (self._link_table_name, self._placeholder(),
@@ -529,7 +532,8 @@ class PostgresqlBackend(object):
                 try:
                     cursor.execute(query, (uuid, source_uuid, uuid, source_uuid))
                 except self._connection._backend.Error as _error:
-                    self._swallow_unique_violation(_error)
+                    if not self._swallow_unique_violation(_error):
+                        raise
                 finally:
                     cursor.close()
 
@@ -697,7 +701,8 @@ class PostgresqlBackend(object):
         type_map[Integer] = "INTEGER"
         type_map[Real] = "DOUBLE PRECISION"
         type_map[Boolean] = "BOOLEAN"
-        type_map[Text] = "TEXT"
+        type_map[Text] = "TEXT COLLATE \"C\""
+        type_map[JSON] = "JSONB"
         type_map[Timestamp] = "TIMESTAMP"
         type_map[UUID] = "UUID"
         type_map[Geometry] = "GEOGRAPHY"
@@ -723,6 +728,11 @@ class PostgresqlBackend(object):
         if tags is None:
             query = "DELETE FROM %s WHERE uuid = %s" % (self._tag_table_name, self._placeholder())
             parameters = (uuid,)
+            cursor = self._connection.cursor()
+            try:
+                cursor.execute(query, parameters)
+            finally:
+                cursor.close()
         else:
             for tag in tags:
                 query = "DELETE FROM %s WHERE uuid = %s AND tag = %s" % (self._tag_table_name,
@@ -759,6 +769,8 @@ class PostgresqlBackend(object):
         # Append the uuid (value) at the end of the list of parameters (will be used in the WHERE clause).
         parameters.append(uuid)
 
+        parameters = [json.dumps(p) if isinstance(p, dict) else p for p in parameters]
+
         # Build and execute UPDATE query.
         set_clause = ", ".join(["%s = %s" % (field, self._placeholder()) for field in fields])
         query = "UPDATE %s SET %s WHERE uuid = %s" % (self._table_name(name), set_clause, self._placeholder())
@@ -785,6 +797,7 @@ class PostgresqlBackend(object):
 
             if value is not None or not schema.is_optional(identifier):
                 unpacked_properties[identifier] = value
+
         return unpacked_properties
 
     def _unpack_product_properties(self, description, values):
@@ -900,7 +913,7 @@ class PostgresqlBackend(object):
         with self._connection:
             self._insert_namespace_properties(properties.core.uuid, "core", properties.core)
             for ns_name, ns_properties in vars(properties).items():
-                if ns_name == "core":
+                if ns_name == "core" or ns_properties is None:
                     continue
                 self._insert_namespace_properties(properties.core.uuid, ns_name, ns_properties)
 
@@ -946,9 +959,10 @@ class PostgresqlBackend(object):
             return self._source_products(uuid)
 
     @translate_errors
-    def summary(self, where="", parameters=None, aggregates=None, group_by=None, group_by_tag=False, order_by=None):
+    def summary(self, where="", parameters=None, aggregates=None, group_by=None, group_by_tag=False,
+                having=None, order_by=None):
         query, query_parameters, query_description = self._sql_builder.build_summary_query(
-            where, parameters, aggregates, group_by, group_by_tag, order_by)
+            where, parameters, aggregates, group_by, group_by_tag, having, order_by)
 
         with self._connection:
             cursor = self._connection.cursor()
@@ -996,7 +1010,8 @@ class PostgresqlBackend(object):
                 if ns_name == "core":
                     continue
                 if ns_name in new_namespaces:
-                    self._insert_namespace_properties(uuid, ns_name, ns_properties)
+                    if ns_properties is not None:
+                        self._insert_namespace_properties(uuid, ns_name, ns_properties)
                 elif ns_properties is None:
                     self._delete_namespace_properties(uuid, ns_name)
                 else:
