@@ -5,17 +5,17 @@
 from __future__ import absolute_import, division, print_function
 
 import collections
+
 try:
     from collections.abc import MutableMapping
 except ImportError:
     from collections import MutableMapping
 
 import inspect
-import re
 
 from muninn.exceptions import *
 from muninn.function import Prototype
-from muninn.language import parse_and_analyze, Literal, Name
+from muninn.language import parse_and_analyze, Literal, Name, Identifier
 from muninn.schema import *
 from muninn.visitor import Visitor
 
@@ -31,12 +31,13 @@ AGGREGATE_FUNCTIONS = collections.OrderedDict([
     # (Geometry, []),
     (None, ['min', 'max', 'sum', 'avg']),  # special case: validity_duration
 ])
+
 GROUP_BY_FUNCTIONS = collections.OrderedDict([
     (Long, [None, ]),
     (Integer, [None, ]),
     # (Real, []),
     (Boolean, [None, ]),
-    (Text, [None]),  # (Text, [None, 'length']),
+    (Text, [None, 'length']),
     (Timestamp, ['year', 'month', 'yearmonth', 'date', 'day', 'hour', 'minute', 'second', 'time']),
     # (UUID, []),
     # (Geometry, []),
@@ -303,9 +304,16 @@ class _WhereExpressionVisitor(Visitor):
         parameter_name = str(self._root_visitor._count)
         self._root_visitor._parameters[parameter_name] = visitable.value
         self._root_visitor._count += 1
-        return self._named_placeholder(parameter_name)
+        return self._named_placeholder(parameter_name, arg=visitable.value)
 
     def visit_Name(self, visitable):
+        if isinstance(visitable.value, Identifier):
+            item = visitable.value
+            if item.canonical == 'count':
+                return 'COUNT(*)'
+            else:
+                return '%s(%s)' % (item.subscript.upper(), self._column_name(item.namespace, item.identifier))
+
         namespace_name = visitable.value.split('.')
         if len(namespace_name) == 1:
             namespace = namespace_name[0]
@@ -320,10 +328,19 @@ class _WhereExpressionVisitor(Visitor):
         return '(' + ','.join(repr(v) for v in visitable.value) + ')'
 
     def visit_ParameterReference(self, visitable):
-        parameter_name = str(self._root_visitor._count)
-        self._root_visitor._parameters[parameter_name] = visitable.value
-        self._root_visitor._count += 1
-        return self._named_placeholder(parameter_name, arg=visitable.value)
+        if visitable.type is Sequence:
+            result = []
+            for value in visitable.value:
+                parameter_name = str(self._root_visitor._count)
+                self._root_visitor._parameters[parameter_name] = value
+                self._root_visitor._count += 1
+                result.append(self._named_placeholder(parameter_name, arg=value))
+            return '(' + ','.join(result) + ')'
+        else:
+            parameter_name = str(self._root_visitor._count)
+            self._root_visitor._parameters[parameter_name] = visitable.value
+            self._root_visitor._count += 1
+            return self._named_placeholder(parameter_name, arg=visitable.value)
 
     def visit_FunctionCall(self, visitable):
         try:
@@ -375,47 +392,6 @@ class _WhereExpressionVisitor(Visitor):
         raise Error("unsupported abstract syntax tree node type: %r" % type(visitable).__name__)
 
 
-class Identifier(object):
-
-    # @staticmethod
-    def __init__(self, canonical_identifier, namespace_schemas):
-        self.canonical = canonical_identifier
-        if canonical_identifier == 'tag':
-            # the rules to get the namespace database table name also apply to 'tag'
-            self.namespace = canonical_identifier
-            self.identifier = canonical_identifier
-            self.subscript = None
-            self.muninn_type = Text
-        elif canonical_identifier == 'count':
-            self.namespace = None
-            self.identifier = canonical_identifier
-            self.subscript = None
-            self.muninn_type = Long
-        elif not re.match(r'[\w]+\.[\w.]+', canonical_identifier):
-            raise Error("cannot resolve identifier: %r" % canonical_identifier)
-        else:
-            split = canonical_identifier.split('.', 2)
-            self.namespace = split[0]
-            self.identifier = split[1]
-            self.subscript = split[2] if len(split) > 2 else None
-            # check if namespace is valid
-            if self.namespace not in namespace_schemas:
-                raise Error("undefined namespace: \"%s\"" % self.namespace)
-            # check if property name is valid
-            if self.identifier not in namespace_schemas[self.namespace]:
-                if self.property_name != 'core.validity_duration':
-                    raise Error("no property: %r defined within namespace: %r" % (self.identifier, self.namespace))
-            # note: not checking if subscript is valid; the list of possible subscripts varies depending on context
-            if self.property_name == 'core.validity_duration':
-                self.muninn_type = None
-            else:
-                self.muninn_type = namespace_schemas[self.namespace][self.identifier]
-
-    @property
-    def property_name(self):
-        return '%s.%s' % (self.namespace, self.identifier)
-
-
 class SQLBuilder(object):
     def __init__(self, namespace_schemas, type_map, rewriter_table, table_name_func, _named_placeholder_func,
                  _placeholder_func, rewriter_property_func):
@@ -467,14 +443,19 @@ class SQLBuilder(object):
         return query, where_parameters
 
     def build_summary_query(self, where='', parameters=None, aggregates=None, group_by=None, group_by_tag=False,
-                            order_by=None):
+                            having=None, order_by=None):
         aggregates = aggregates or []
         group_by = group_by or []
         order_by = order_by or []
         if group_by_tag:
             group_by = group_by + ['tag']
-        result_fields = group_by + ['count'] + aggregates
-        join_set = set(item.split('.')[0] for item in group_by)
+
+        join_set = set()
+        result_fields = []
+        for field in group_by + ['count'] + aggregates:
+            ident = Identifier(field, self._namespace_schemas)
+            result_fields.append(ident.resolve)
+            join_set.add(ident.namespace)
 
         # Parse the WHERE clause.
         where_clause, where_parameters = '', {}
@@ -492,13 +473,23 @@ class SQLBuilder(object):
         if group_by_list:
             group_by_clause = 'GROUP BY %s' % ', '.join(group_by_list)
 
+        # Generate the HAVING clause
+        having_clause = ''
+        if having is not None:
+            ast = parse_and_analyze(having, self._namespace_schemas, parameters, having=True)
+            visitor = _WhereExpressionVisitor(self._rewriter_table, self._column_name, self._named_placeholder,
+                                              visitor if where else None)
+            having_expr, having_parameters, _ = visitor.do_visit(ast)
+            where_parameters.update(having_parameters)
+            having_clause = 'HAVING %s' % having_expr
+
         # Parse the ORDER BY clause.
         order_by_clause = ''
         order_by_list = []
         for item in order_by:
             direction = 'DESC' if item.startswith('-') else 'ASC'
             name = item[1:] if item.startswith('+') or item.startswith('-') else item
-            Identifier(name, self._namespace_schemas)  # check if the identifier is valid
+            name = Identifier(name, self._namespace_schemas).resolve
             if name not in result_fields:
                 raise Error("cannot order result by %r; field is not present in result" % name)
             order_by_list.append('"%s" %s' % (name, direction))
@@ -532,7 +523,8 @@ class SQLBuilder(object):
                                                                   group_by_functions))
             if item.subscript:
                 column_name = self._rewriter_property(column_name, item.subscript)
-            select_list.append('%s AS "%s"' % (column_name, item.canonical))
+            select_list.append('%s AS "%s"' % (column_name, item.resolve))
+
         # aggregated fields
         select_list.append('COUNT(*) AS count')  # always aggregate row count
         for item in aggregates:
@@ -562,9 +554,9 @@ class SQLBuilder(object):
         # Generate the FROM clause.
         from_clause = 'FROM %s' % self._table_name('core')
 
-        join_set.discard('core')
         for namespace in join_set:
-            from_clause = '%s LEFT JOIN %s USING (uuid)' % (from_clause, self._table_name(namespace))
+            if namespace not in ('core', None):
+                from_clause = '%s LEFT JOIN %s USING (uuid)' % (from_clause, self._table_name(namespace))
 
         # Generate the complete query.
         query = '%s\n%s' % (select_clause, from_clause)
@@ -572,6 +564,8 @@ class SQLBuilder(object):
             query = '%s\n%s' % (query, where_clause)
         if group_by_clause:
             query = '%s\n%s' % (query, group_by_clause)
+        if having_clause:
+            query = '%s\n%s' % (query, having_clause)
         if order_by_clause:
             query = '%s\n%s' % (query, order_by_clause)
 
@@ -660,9 +654,12 @@ class SQLBuilder(object):
             direction = "DESC" if item.startswith("-") else "ASC"
             name = item[1:] if item.startswith("+") or item.startswith("-") else item
 
-            try:
+            segments = name.split('.')
+            if len(segments) == 1:
+                namespace, name = 'core', segments[0]
+            elif len(segments) == 2:
                 namespace, name = name.split(".")
-            except ValueError:
+            else:
                 raise Error("invalid property name: %r" % name)
 
             if name not in self._namespace_schema(namespace):
