@@ -9,6 +9,7 @@ import os
 import re
 import datetime
 import functools
+import json
 import uuid
 
 # Select a version of dbapi2 that's available.
@@ -364,8 +365,8 @@ class SQLiteBackend(object):
     def _execute_list(self, sql_list):
         cursor = self._connection.cursor()
         try:
-            for sql in sql_list:
-                cursor.execute(sql)
+            for item in sql_list:
+                cursor.execute(item)
         finally:
             cursor.close()
 
@@ -434,6 +435,8 @@ class SQLiteBackend(object):
             fields.append("uuid")
             parameters.append(uuid)
 
+        parameters = [json.dumps(p) if isinstance(p, dict) else p for p in parameters]
+
         # Build and execute INSERT query.
         query = "INSERT INTO %s (%s) VALUES (%s)" % (self._table_name(name), ", ".join(fields),
                                                      ", ".join([self._placeholder()] * len(fields)))
@@ -630,6 +633,7 @@ class SQLiteBackend(object):
         type_map[Timestamp] = "TIMESTAMP"
         type_map[UUID] = "UUID"
         type_map[Geometry] = "GEOMETRY"
+        type_map[JSON] = "TEXT"
 
         return type_map
 
@@ -639,7 +643,7 @@ class SQLiteBackend(object):
             parameters = (uuid,)
         else:
             query = "DELETE FROM %s WHERE uuid = %s AND source_uuid IN (%s)" % \
-                (self._link_table_name, self._placeholder(), ','.join(self._placeholder() * len(tags)))
+                (self._link_table_name, self._placeholder(), ','.join(self._placeholder() * len(source_uuids)))
             parameters = [uuid] + list(source_uuids)
 
         cursor = self._connection.cursor()
@@ -681,6 +685,8 @@ class SQLiteBackend(object):
         schema = self._namespace_schema(namespace)
         for identifier, value in zip(description, values):
             if value is not None or not schema.is_optional(identifier):
+                if issubclass(schema[identifier], JSON):
+                    value = json.loads(value)
                 unpacked_properties[identifier] = value
         return unpacked_properties
 
@@ -724,6 +730,8 @@ class SQLiteBackend(object):
         # Append the uuid (value) at the end of the list of parameters (will be used in the WHERE clause).
         parameters.append(uuid)
 
+        parameters = [json.dumps(p) if isinstance(p, dict) else p for p in parameters]
+
         # Build and execute UPDATE query.
         set_clause = ", ".join(["%s = %s" % (field, self._placeholder()) for field in fields])
         query = "UPDATE %s SET %s WHERE uuid = %s" % (self._table_name(name), set_clause, self._placeholder())
@@ -825,7 +833,7 @@ class SQLiteBackend(object):
         with self._connection:
             self._insert_namespace_properties(properties.core.uuid, "core", properties.core)
             for ns_name, ns_properties in vars(properties).items():
-                if ns_name == "core":
+                if ns_name == "core" or ns_properties is None:
                     continue
                 self._insert_namespace_properties(properties.core.uuid, ns_name, ns_properties)
 
@@ -869,9 +877,10 @@ class SQLiteBackend(object):
             return self._source_products(uuid)
 
     @translate_sqlite_errors
-    def summary(self, where="", parameters=None, aggregates=None, group_by=None, group_by_tag=False, order_by=None):
+    def summary(self, where="", parameters=None, aggregates=None, group_by=None, group_by_tag=False,
+                having=None, order_by=None):
         query, query_parameters, query_description = self._sql_builder.build_summary_query(
-            where, parameters, aggregates, group_by, group_by_tag, order_by)
+            where, parameters, aggregates, group_by, group_by_tag, having, order_by)
 
         with self._connection:
             cursor = self._connection.cursor()
@@ -909,7 +918,8 @@ class SQLiteBackend(object):
                 if ns_name == "core":
                     continue
                 if ns_name in new_namespaces:
-                    self._insert_namespace_properties(uuid, ns_name, ns_properties)
+                    if ns_properties is not None:
+                        self._insert_namespace_properties(uuid, ns_name, ns_properties)
                 elif ns_properties is None:
                     self._delete_namespace_properties(uuid, ns_name)
                 else:
