@@ -19,6 +19,7 @@ from muninn.function import Prototype, FunctionTable
 from muninn.schema import *
 from muninn.visitor import Visitor
 
+
 #
 # Table of all supported operators and functions
 #
@@ -218,7 +219,7 @@ class TokenStream(object):
             r"""\d{4}-\d{2}-\d{2}(?:T\d{2}:\d{2}:\d{2}(?:\.\d{0,6})?)?""",   # Timestamp literals
             r"""[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}""",  # UUID literals
             r"""\d+(?:\.\d*(?:[eE][+-]?\d+)?|[eE][+-]?\d+)""",               # Real literals
-            r"""\d+""",                                                      # Integer literals
+            r"""0x[0-9a-fA-F]+|0o[0-7]\d+|0b[0-1]+|\d+""",                   # Integer literals
             r"""<=|>=|==|!=|~=|not in|[*<>@()\[\],.+-/]""",                  # Operators and delimiters
             r"""[a-zA-Z]\w*""",                                              # Names (incl. true, false, in)
         )
@@ -331,7 +332,15 @@ class TokenStream(object):
             return Token(TokenType.REAL, float(real))
 
         if integer is not None:
-            return Token(TokenType.INTEGER, int(integer))
+            if integer.startswith('0x'):
+                base = 16
+            elif integer.startswith('0o'):
+                base = 8
+            elif integer.startswith('0b'):
+                base = 2
+            else:
+                base = 10
+            return Token(TokenType.INTEGER, int(integer, base))
 
         if operator is not None:
             return Token(TokenType.OPERATOR, operator)
@@ -592,6 +601,9 @@ def parse_expression(stream):
 
 
 def _literal_type(literal):
+    if isinstance(literal, list):  # TODO use Sequence.validate..?
+        return Sequence
+
     for type in (Text, Timestamp, UUID, Boolean, Integer, Long, Real, Geometry):
         try:
             type.validate(literal)
@@ -604,15 +616,22 @@ def _literal_type(literal):
 
 
 class SemanticAnalysis(Visitor):
-    def __init__(self, namespace_schemas, parameters):
+    def __init__(self, namespace_schemas, parameters, having=False):
         super(SemanticAnalysis, self).__init__()
         self._namespace_schemas = namespace_schemas
         self._parameters = parameters
+        self._having = having
 
     def visit_Literal(self, visitable):
         visitable.type = _literal_type(visitable.value)
 
     def visit_Name(self, visitable):
+        if self._having:
+            item = Identifier(visitable.value, self._namespace_schemas)
+            visitable.type = item.muninn_type
+            visitable.value = item
+            return
+
         split_name = visitable.value.split(".")
 
         # namespace/implicit core property
@@ -712,14 +731,14 @@ def parse(text):
     return abstract_syntax_tree
 
 
-def analyze(abstract_syntax_tree, namespace_schemas={}, parameters={}):
+def analyze(abstract_syntax_tree, namespace_schemas={}, parameters={}, having=False):
     annotated_syntax_tree = copy.deepcopy(abstract_syntax_tree)
-    SemanticAnalysis(namespace_schemas, parameters).visit(annotated_syntax_tree)
+    SemanticAnalysis(namespace_schemas, parameters, having=having).visit(annotated_syntax_tree)
     return annotated_syntax_tree
 
 
-def parse_and_analyze(text, namespace_schemas={}, parameters={}):
-    return analyze(parse(text), namespace_schemas, parameters)
+def parse_and_analyze(text, namespace_schemas={}, parameters={}, having=False):
+    return analyze(parse(text), namespace_schemas, parameters, having)
 
 
 def string_unescape(text):
@@ -749,3 +768,71 @@ def string_unescape(text):
 
     result = regex.sub(_replace, text)
     return result
+
+
+class Identifier(object):
+    def __init__(self, canonical_identifier, namespace_schemas):
+        self.canonical = canonical_identifier
+        self.subscript = None
+
+        if canonical_identifier == 'tag':
+            # the rules to get the namespace database table name also apply to 'tag'
+            self.namespace = canonical_identifier
+            self.identifier = canonical_identifier
+            self.muninn_type = Text
+
+        elif canonical_identifier == 'count':
+            self.namespace = None
+            self.identifier = canonical_identifier
+            self.muninn_type = Long
+
+        elif not re.match(r'[\w]+\.[\w.]+', canonical_identifier):
+            raise Error("cannot resolve identifier: %r" % canonical_identifier)
+
+        else:
+            segments = canonical_identifier.split('.')
+
+            if len(segments) == 1:
+                self.namespace = 'core'
+                self.identifier = segments[0]
+
+            elif len(segments) == 2:
+                if segments[0] in namespace_schemas:
+                    self.namespace, self.identifier = segments
+                else:
+                    self.namespace = 'core'
+                    self.identifier, self.subscript = segments
+
+            elif len(segments) == 3:
+                self.namespace, self.identifier, self.subscript = segments
+
+            else:
+                raise Error("cannot resolve identifier: %r" % canonical_identifier)
+
+            # check if namespace is valid
+            if self.namespace not in namespace_schemas:
+                raise Error("undefined namespace: \"%s\"" % self.namespace)
+
+            # check if property name is valid
+            if self.identifier not in namespace_schemas[self.namespace]:
+                if self.property_name != 'core.validity_duration':
+                    raise Error("no property: %r defined within namespace: %r" % (self.identifier, self.namespace))
+
+            # note: not checking if subscript is valid; the list of possible subscripts varies depending on context
+            if self.property_name == 'core.validity_duration':
+                self.muninn_type = None
+            else:
+                self.muninn_type = namespace_schemas[self.namespace][self.identifier]
+
+    @property
+    def property_name(self):
+        return '%s.%s' % (self.namespace, self.identifier)
+
+    @property
+    def resolve(self):
+        if self.canonical in ('count', 'tag'):
+            return self.canonical
+        elif self.subscript is None:
+            return '%s.%s' % (self.namespace, self.identifier)
+        else:
+            return '%s.%s.%s' % (self.namespace, self.identifier, self.subscript)
