@@ -20,53 +20,75 @@ class _S3Config(Mapping):
     _alias = "s3"
 
     host = Text()
-    port = Integer()
+    port = Integer(optional=True)
     bucket = Text()
-    access_key = Text()
-    secret_access_key = Text()
+    access_key = Text(optional=True)
+    secret_access_key = Text(optional=True)
     region = Text(optional=True)
     prefix = Text(optional=True)
-    tmp_root = Text(optional=True)
     download_args = Text(optional=True)  # JSON representation of boto3 download_file ExtraArgs parameter
     upload_args = Text(optional=True)  # JSON representation of boto3 upload_file ExtraArgs parameter
     copy_args = Text(optional=True)  # JSON representation of boto3 copy ExtraArgs parameter
     transfer_config = Text(optional=True)  # JSON representation of boto3.s3.transfer.TransferConfig parameters
 
 
-def create(configuration):
+def create(configuration, tempdir, auth_file):
     options = config.parse(configuration.get("s3", {}), _S3Config)
+
+    # if access_key and secret_access_key missing, use auth_file
+    if (auth_file is not None and
+            'access_key' not in options and
+            'secret_access_key' not in options and
+            'host' in options and
+            'bucket' in options):
+        credentials = json.loads(open(auth_file).read())
+        for key, value in credentials.items():
+            if value.get('auth_type') == 'S3' and key == options['host'] and value.get('bucket') == options['bucket']:
+                for option in ('access_key', 'secret_access_key', 'port', 'bucket', 'region'):
+                    if option in value and option not in options:
+                        options[option] = value[option]
+                break
+
+    # check that mandatory options are configured
+    for option in ('access_key', 'secret_access_key'):
+        if option not in options:
+            raise Error("'%s' not configured" % option)
+
     _S3Config.validate(options)
-    return S3StorageBackend(**options)
+    return S3StorageBackend(tempdir=tempdir, **options)
 
 
 class S3StorageBackend(StorageBackend):  # TODO '/' in keys to indicate directory, 'dir/' with contents?
-    def __init__(self, bucket, host, port, access_key, secret_access_key, region=None, prefix='', tmp_root=None,
-                 download_args=None, upload_args=None, copy_args=None, transfer_config=None):
-        super(S3StorageBackend, self).__init__()
+    def __init__(self, bucket, host, access_key, secret_access_key, port=None, region=None, prefix='',
+                 download_args=None, upload_args=None, copy_args=None, transfer_config=None, tempdir=None):
+        super(S3StorageBackend, self).__init__(tempdir)
 
         self.bucket = bucket
+
         if prefix and not prefix.endswith('/'):
             prefix += '/'
         self._prefix = prefix
 
-        if port == 80:
-            export_port = ''
-        else:
-            export_port = ':%d' % port
-        self.global_prefix = os.path.join('http://%s%s/%s' % (host, export_port, bucket), prefix)
+        endpoint_url = host
+        if ':' not in host:
+            if port == 443:
+                endpoint_url = 'https://' + endpoint_url
+            else:
+                endpoint_url = 'http://' + endpoint_url
+                if port is not None and port != 80:
+                    endpoint_url += ':%d' % port
+        elif port is not None:
+            endpoint_url += ':%d' % port
+        self.global_prefix = os.path.join(endpoint_url, bucket, prefix)
 
         self._root = bucket
-        if tmp_root:
-            tmp_root = os.path.realpath(tmp_root)
-            util.make_path(tmp_root)
-        self._tmp_root = tmp_root
 
         self._resource = boto3.resource(
             service_name='s3',
             region_name=region,
             aws_access_key_id=access_key,
             aws_secret_access_key=secret_access_key,
-            endpoint_url='http://%s:%s' % (host, port),
+            endpoint_url=endpoint_url,
         )
 
         self._download_args = None
@@ -118,7 +140,7 @@ class S3StorageBackend(StorageBackend):  # TODO '/' in keys to indicate director
     def product_path(self, product):  # TODO needed?
         return os.path.join(product.core.archive_path, product.core.physical_name)
 
-    def current_archive_path(self, paths):
+    def current_archive_path(self, paths, properties):
         raise Error("S3 storage backend does not support ingesting already archived products")
 
     def _upload_file(self, key, path):
