@@ -1,3 +1,4 @@
+import json
 import logging
 import os
 
@@ -16,29 +17,40 @@ logging.getLogger("swiftclient").setLevel(logging.CRITICAL)
 class _SwiftConfig(Mapping):
     _alias = "swift"
 
-    container = Text()
-    user = Text()
-    key = Text()
     authurl = Text()
-    tmp_root = Text(optional=True)
+    container = Text(optional=True)
+    user = Text(optional=True)
+    key = Text(optional=True)
 
 
-def create(configuration):
+def create(configuration, tempdir, auth_file):
     options = config.parse(configuration.get("swift", {}), _SwiftConfig)
+
+    # if access_key and secret_access_key missing, use auth_file
+    if 'user' not in options and 'key' not in options and auth_file is not None:
+        credentials = json.loads(open(auth_file).read())
+        for key, value in credentials.items():
+            if key == options['authurl'] and value.get('auth_type') == 'Swift':
+                for option in ('user', 'key', 'container'):
+                    if option in value and option not in options:
+                        options[option] = value[option]
+                break
+
+    # check that mandatory options are configured
+    for option in ('user', 'key', 'container'):
+        if option not in options:
+            raise Error("'%s' not configured" % option)
+
     _SwiftConfig.validate(options)
-    return SwiftStorageBackend(**options)
+    return SwiftStorageBackend(tempdir=tempdir, **options)
 
 
 class SwiftStorageBackend(StorageBackend):  # TODO '/' in keys to indicate directory, 'dir/' with contents?
-    def __init__(self, container, user, key, authurl, tmp_root=None):
-        super(SwiftStorageBackend, self).__init__()
+    def __init__(self, container, user, key, authurl, tempdir=None):
+        super(SwiftStorageBackend, self).__init__(tempdir)
 
         self.container = container
         self._root = container
-        if tmp_root:
-            tmp_root = os.path.realpath(tmp_root)
-            util.make_path(tmp_root)
-        self._tmp_root = tmp_root
 
         self._conn = swiftclient.Connection(
             user=user,
@@ -73,7 +85,7 @@ class SwiftStorageBackend(StorageBackend):  # TODO '/' in keys to indicate direc
     def product_path(self, product):  # TODO needed?
         return os.path.join(product.core.archive_path, product.core.physical_name)
 
-    def current_archive_path(self, paths):
+    def current_archive_path(self, paths, properties):
         raise Error("Swift storage backend does not support ingesting already archived products")
 
     def put(self, paths, properties, use_enclosing_directory, use_symlinks=None,
