@@ -6,7 +6,7 @@ from __future__ import absolute_import, division, print_function
 
 import muninn
 
-from .utils import create_parser, parse_args_and_run
+from muninn.tools.utils import create_parser, parse_args_and_run
 
 
 def list_tags(args):
@@ -24,3 +24,7 @@ def main():
     parser.add_argument("expression", metavar="EXPRESSION",
                         help="expression used to search for products to list tags of")
     return parse_args_and_run(parser, list_tags)
+
+
+if __name__ == '__main__':
+    main()
