@@ -17,15 +17,13 @@ except ImportError:
 import muninn
 from muninn.struct import Struct
 
-from .utils import Processor, create_parser, parse_args_and_run
+from muninn.tools.utils import Processor, create_parser, parse_args_and_run
 
 logger = logging.getLogger(__name__)
 
 ACTIONS = [
     'ingest',
-    'post_ingest',
     'pull',
-    'post_pull',
     'retype',
 ]
 
@@ -33,7 +31,7 @@ ACTIONS = [
 class UpdateProcessor(Processor):
 
     def __init__(self, args):
-        super(UpdateProcessor, self).__init__(args.archive)
+        super(UpdateProcessor, self).__init__(args)
         self.action = args.action
         self.argument = args.argument
         self.disable_hooks = args.disable_hooks
@@ -46,24 +44,12 @@ class UpdateProcessor(Processor):
             archive.rebuild_properties(product.core.uuid, disable_hooks=self.disable_hooks,
                                        use_current_path=self.use_current_path)
 
-        elif self.action == 'post_ingest':
-            plugin = archive.product_type_plugin(product.core.product_type)
-            if hasattr(plugin, "post_ingest_hook"):
-                logger.debug('running update:post_ingest on %s ' % product.core.product_name)
-                plugin.post_ingest_hook(archive, product)
-
         elif self.action == 'pull':
             logger.debug('running update:pull on %s ' % product.core.product_name)
             archive.rebuild_pull_properties(product.core.uuid, verify_hash=self.verify_hash,
                                             disable_hooks=self.disable_hooks,
                                             use_current_path=self.use_current_path)
 
-        elif self.action == 'post_pull':
-            plugin = archive.product_type_plugin(product.core.product_type)
-            if hasattr(plugin, "post_pull_hook"):
-                logger.debug('running update:post_pull on %s ' % product.core.product_name)
-                plugin.post_pull_hook(archive, product)
-
         elif self.action == 'retype':
             if self.argument is not None:
                 archive.update_properties(Struct({'core': {'product_type': self.argument}}), product.core.uuid)
@@ -73,16 +59,19 @@ class UpdateProcessor(Processor):
 
 
 def update(args):
-    expression = "is_defined(core.archive_path)"
-    if args.expression:
-        expression += " and (%s)" % args.expression
-
+    filter = []
+    if args.action in ['ingest', 'pull']:
+        # only get products that are locally archived
+        filter.append("is_defined(core.archive_path)")
     if args.action == 'pull':
         # only get products with a remote_url
-        if expression:
-            expression = "is_defined(remote_url) and (%s)" % expression
+        filter.append("is_defined(remote_url)")
+    if args.expression:
+        if len(filter) == 0:
+            filter.append(args.expression)
         else:
-            expression = "is_defined(remote_url)"
+            filter.append("(%s)" % args.expression)
+    expression = " and ".join(filter)
 
     processor = UpdateProcessor(args)
     with muninn.open(args.archive) as archive:
@@ -109,11 +98,9 @@ def update(args):
 def main():
     parser = create_parser(description="""Updates properties of existing products.
         This is an archive maintenance tool, meant to be used when the archive structure has changed.
-        Use with care!""")
+        Use with care!""", parallel=True)
     parser.add_argument("--disable-hooks", action="store_true",
                         help="do not run the hooks associated with the action")
-    parser.add_argument("--parallel", action="store_true", help="use multi-processing to perform update")
-    parser.add_argument("--processes", type=int, help="use a specific amount of processes for --parallel")
     parser.add_argument("--verify-hash", action="store_true",
                         help="verify the hash of the product after a `pull` update")
     parser.add_argument("-k", "--keep", action="store_true",
@@ -124,3 +111,7 @@ def main():
     parser.add_argument("expression", metavar="EXPRESSION", default="", help="expression to select products")
     parser.add_argument("argument", metavar="ARGUMENT", nargs='?', help="product type for retype action")
     return parse_args_and_run(parser, update)
+
+
+if __name__ == '__main__':
+    main()
