@@ -6,10 +6,20 @@ from __future__ import absolute_import, division, print_function
 
 import argparse
 import logging
-import muninn
+import multiprocessing
 import os
 import sys
 
+try:
+    from tqdm import tqdm as bar
+    from multiprocessing import RLock
+    RLock()  # fork off resource tracker before tqdm does it (with multiple threads running)
+except ImportError:
+    def bar(range, total=None):
+        return range
+
+import muninn
+
 
 # This is a base class for operations on a list of items that can be performed in parallel using multiprocessing.
 # If you use the processor object as a callable then it is assumed that the operation is performed using subprocesses.
@@ -18,10 +28,18 @@ import sys
 # archive handle.
 class Processor(object):
 
-    def __init__(self, archive_name):
-        self._archive_name = archive_name
+    def __init__(self, args):
+        global _POOL  # TODO what about multiple processors..
+
+        self._archive_name = args.archive
         self._archive = None
 
+        if args.parallel:
+            if args.processes is not None:
+                _POOL = multiprocessing.Pool(args.processes)
+            else:
+                _POOL = multiprocessing.Pool()
+
     def perform_operation(self, archive, item):
         pass
 
@@ -34,6 +52,25 @@ class Processor(object):
             # don't capture keyboard interrupts inside sub-processes (only the main process should handle it)
             pass
 
+    def process(self, archive, args, items):
+        total = len(items)
+        num_success = 0
+
+        if args.parallel:
+            num_success = sum(list(bar(_POOL.imap(self, items), total=total)))
+            _POOL.close()
+            _POOL.join()
+
+        elif total > 1:
+            for item in bar(items):
+                num_success += self.perform_operation(archive, item)
+
+        elif total == 1:
+            # don't show progress bar if we ingest just one item
+            num_success = self.perform_operation(archive, items[0])
+
+        return 0 if num_success == total else 1
+
 
 # This parser is used in combination with the parse_known_args() function as a way to implement a "--version"
 # option that prints version information and exits, and is included in the help message.
@@ -46,8 +83,17 @@ version_parser.add_argument("--version", action="store_true", help="output versi
 
 
 def create_parser(*args, **kwargs):
+    parallel = False
+    if 'parallel' in kwargs:
+        parallel = kwargs.pop('parallel')
+
     parser = argparse.ArgumentParser(*args, parents=[version_parser], **kwargs)
     parser.add_argument("--verbose", action="store_true", help="display debug information")
+
+    if parallel:
+        parser.add_argument("--parallel", action="store_true", help="use multi-processing to perform operation")
+        parser.add_argument("--processes", type=int, help="use a specific amount of processes for --parallel")
+
     return parser
 
 
